/*
 *
 * Copyright  :: 1995-2006 (C)opyright Gordano Ltd
 *               All rights reserved.
 *               No copies of this source may be made without express written
 *               premission of Gordano Ltd, 18 Kenn Road, Clevedon, BS21 6EL
 *
 * $Header: d:\source\data\ntmail17_01\optpack\efmfull/efm.h,v 1.2 2006/05/25 10:53:24 simont Exp $
 *
 * $Log: efm.h,v $
 * Revision 1.2  2006/05/25  10:53:24  simont
 * Added new API
 *
 * Revision 1.1  2006/01/18  08:18:29  simont
 * Initial revision
 *
 *
 *
 */

#ifndef _EFM_H
#define _EFM_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Definitions
 */
#define EFM_FOLDER_OPEN_FULL            0
#define EFM_FOLDER_OPEN_INFORMATION     1

#define EFM_FOLDER_CLOSE_FULL           0
#define EFM_FOLDER_CLOSE_INFORMATION    1

#define EFM_SERVICE_TYPE_UNKNOWN        0
#define EFM_SERVICE_TYPE_SMTP           1
#define EFM_SERVICE_TYPE_POP            2
#define EFM_SERVICE_TYPE_IMAP           3
#define EFM_SERVICE_TYPE_WWW            4
#define EFM_SERVICE_TYPE_MESSENGER      5

#define EFM_EVENT_UNUSED                0
#define EFM_EVENT_NEW                   1
#define EFM_EVENT_FLAGS                 2
#define EFM_EVENT_EXPUNGE               3

/*
 * Type definitions
 */
#define EFM_INVALID_SESSION_HANDLE      0
#define EFM_INVALID_LOCK_HANDLE         0
#define EFM_INVALID_FOLDER_HANDLE       0

typedef long EFM_SESSION_HANDLE;
typedef long EFM_LOCK_HANDLE;
typedef long EFM_FOLDER_HANDLE;

#define EFM_FOLDER_LIST_NO_SELECT           1
#define EFM_FOLDER_LIST_NO_SELECT_BIT       0

#define EFM_FOLDER_LIST_NO_INFERIORS        2
#define EFM_FOLDER_LIST_NO_INFERIORS_BIT    1

#define EFM_MAX_UDIL                64
#define EFM_MAX_FLAGS               512
#define EFM_MAX_EMAIL_ADDRESS       256
#define EFM_MAX_FILENAME            256

typedef struct efm_message_list_struct
{
    int  number;
    char udil[EFM_MAX_UDIL];
    unsigned int uid;
    unsigned long  size;
    char flags[EFM_MAX_FLAGS];
    int event;

    struct efm_message_list_struct *next;
} EFM_MESSAGE_LIST;

typedef struct efm_folder_list_struct
{
    char *foldername;
    int attributes;

    struct efm_folder_list_struct *next;
} EFM_FOLDER_LIST;

typedef int (*LPFN_EFM_UPDATE_CALLBACK_FN)
(
    char *user,
    char *domain,
    char *foldername,
    EFM_MESSAGE_LIST *list
);

/*
 * Globals
 */


/*
 * Functions
 */

/*
 * Configuration
 */
extern int EFMConfigure( int id,
                         void *logMes,
                         void *logHandle );


extern int EFMClose( int id );

/*
 * Sessions
 */
extern int EFMCreateSession(int id,
                            char *user, char *domain,
                            int serviceType,
                            EFM_SESSION_HANDLE *sessionHandle);

extern int EFMCloseSession(int id,
                           EFM_SESSION_HANDLE *sessionHandle);

extern int EFMKeepAlive(int id,
                        EFM_SESSION_HANDLE sessionHandle);

extern int EFMExpungedMessageAccessAllowed(int id,
                                           EFM_SESSION_HANDLE hFMSession,
                                           BOOL *allowed);

/*
 * Messages
 */
extern int EFMGetMessage(int id,
                         EFM_FOLDER_HANDLE folderHandle,
                         unsigned int uid,
                         char *filename, size_t size);

extern int EFMCloseMessage(int id,
                           EFM_FOLDER_HANDLE folderHandle,
                           unsigned int uid,
                           char *filename);

extern int EFMCopyMessages(int id,
                           EFM_FOLDER_HANDLE folderHandle,
                           char *foldername,
                           EFM_MESSAGE_LIST *messageList);

extern int EFMSetMessageFlags(int id,
                              EFM_FOLDER_HANDLE folderHandle,
                              unsigned int uid,
                              char *flags);

/*
 * Folders
 */
extern int EFMIsExternalFolder(int id,
                               char *user, char *domain,
                               int serviceType,
                               BOOL *external);

extern int EFMGetFolderUIDValidity(int id,
                                   EFM_SESSION_HANDLE sessionHandle,
                                   char *foldername,
                                   unsigned int *uidvalidity);

extern int EFMGetFolderNextUID(int id,
                               EFM_SESSION_HANDLE sessionHandle,
                               char *foldername,
                               unsigned int *uid);

extern int EFMExistFolder(int id,
                          EFM_SESSION_HANDLE sessionHandle,
                          char *foldername,
                          BOOL *exists);

extern int EFMCreateFolder(int id,
                           EFM_SESSION_HANDLE sessionHandle,
                           char *foldername);

extern int EFMDeleteFolder(int id,
                           EFM_SESSION_HANDLE sessionHandle,
                           char *foldername);

extern int EFMGetFolderList(int id,
                            EFM_SESSION_HANDLE sessionHandle,
                            EFM_FOLDER_LIST **list);

extern int EFMCloseFolderList(int id,
                              EFM_SESSION_HANDLE sessionHandle,
                              EFM_FOLDER_LIST **list);

extern int EFMGetFolderModified(int id,
                                EFM_SESSION_HANDLE sessionHandle,
                                char *folderName,
                                char *modifiedStr, int size);

extern int EFMGetFolderSize(int id,
                            EFM_SESSION_HANDLE sessionHandle,
                            char *folderName,
                            DWORD *size);

extern int EFMFolderCountMessagesOfType(int id,
                                        EFM_SESSION_HANDLE sessionHandle,
                                        char *foldername,
                                        char *includeFlags,
                                        char *excludeFlags,
                                        int *messageCount);

extern int EFMRenameFolder(int id,
                           EFM_SESSION_HANDLE sessionHandle,
                           char *foldername,
                           char *newfoldername,
                           BOOL renameSubFolders);

extern int EFMOpenFolder(int id,
                         EFM_SESSION_HANDLE sessionHandle,
                         char *foldername,
                         int flags,
                         EFM_FOLDER_HANDLE *folderHandle);

extern int EFMGetFolderIndex(int id,
                             EFM_FOLDER_HANDLE folderHandle,
                             EFM_MESSAGE_LIST **messageList);

extern int EFMCloseFolderIndex(int id,
                               EFM_FOLDER_HANDLE folderHandle,
                               EFM_MESSAGE_LIST **messageList);

extern int EFMCloseFolder(int id,
                          EFM_SESSION_HANDLE sessionHandle,
                          EFM_FOLDER_HANDLE *folderHandle,
                          int flags);

extern int EFMCommitFolder(int id,
                           EFM_FOLDER_HANDLE folderHandle);

extern int EFMPurgeFolder(int id,
                          EFM_FOLDER_HANDLE folderHandle);

extern int EFMStoreFolder(int id,
                          EFM_SESSION_HANDLE sessionHandle,
                          char *foldername,
                          char *filename,
                          char *flags,
                          unsigned int *uid,
                          char *udil, int size);

/*
 * Folder Locking
 */
extern int EFMClientFolderLock(int id,
                               EFM_SESSION_HANDLE sessionHandle,
                               char *foldername,
                               EFM_LOCK_HANDLE *lockHandle);

extern int EFMServiceFolderLock(int id,
                                EFM_SESSION_HANDLE sessionHandle,
                                char *foldername,
                                EFM_LOCK_HANDLE *lockHandle);

extern int EFMCloseFolderLock(int id,
                              EFM_LOCK_HANDLE *lockHandle);

extern int EFMGMSClientFolderLockRequired(int id,
                                          BOOL *required);

extern int EFMGMSServiceFolderLockRequired(int id,
                                           BOOL *required);

extern int EFMDoesClientFolderLockExist(int id,
                                        EFM_SESSION_HANDLE sessionHandle,
                                        char *foldername,
                                        BOOL *exists);

extern int EFMDoesServiceFolderLockExist(int id,
                                         EFM_SESSION_HANDLE sessionHandle,
                                         char *foldername,
                                         BOOL *exists);

/*
 * Folder Subscription
 */
extern int EFMSubscribeFolder(int id,
                              EFM_SESSION_HANDLE sessionHandle,
                              char *foldername);

extern int EFMUnsubscribeFolder(int id,
                                EFM_SESSION_HANDLE sessionHandle,
                                char *foldername);

extern int EFMGetSubscribedFolderList(int id,
                                      EFM_SESSION_HANDLE sessionHandle,
                                      EFM_FOLDER_LIST **list);

extern int EFMCloseSubscribedFolderList(int id,
                                        EFM_SESSION_HANDLE sessionHandle,
                                        EFM_FOLDER_LIST **list);

/*
 * Update callback
 */
extern int EFMRegisterUpdateCallback(int id,
                                     LPFN_EFM_UPDATE_CALLBACK_FN updateCallback);


#ifdef __cplusplus
}
#endif

#endif
