/*
 *
 * Copyright  :: 1995-1999 (C)opyright Gordano Ltd
 *               All rights reserved.
 *               No copies of this source may be made without express written
 *               permission of Gordano Ltd, PO Box 79, Clevedon,
 *               North Somerset, BS21 7TQ, UK
 *
 * $Header: d:\source\data\ntmail17_01\optpack\smtpdll/mail_dll_type.h,v 1.3 2004/02/04 09:58:07 garrardc Exp $
 *
 * $Log: mail_dll_type.h,v $
 * Revision 1.3  2004/02/04  09:58:07  garrardc
 * OR 07006 - no autoresponse via mailman if virtual
 *
 * Revision 1.2  2002/04/16  14:29:54  simont
 * Messsage logging for SMS
 *
 * Revision 1.1  1999/09/15  16:59:13  garrardc
 * Initial revision
 *
 *
 */

#ifndef _MAIL_DLL_TYPE_H
#define _MAIL_DLL_TYPE_H

#include <windows.h>

#include "constant.h"

#ifdef __cplusplus
extern "C" {
#endif

#define MAX_MAIL_DESCRIPTION    1024

typedef void *MAIL_HANDLE;

typedef enum
{
    MAIL_TYPE_INCOMING  = 1,
    MAIL_TYPE_OUTGOING  = 2,
    MAIL_TYPE_DELIVERY  = 4,
    MAIL_TYPE_ROBOT     = 8,
    MAIL_TYPE_DOMAIN    = 16
} MAIL_TYPE;

typedef struct mail_version_struct
{
    DWORD version;
    MAIL_TYPE dllType;
    MAIL_HANDLE context;
    char description[ MAX_MAIL_DESCRIPTION ];
    char copyright[ MAX_MAIL_DESCRIPTION ];
} MAIL_VERSION;

typedef enum
{
    MAIL_STATUS_STAT        = 1,
    MAIL_STATUS_FAILURE     = 2,
    MAIL_STATUS_DEBUG       = 4,
    MAIL_STATUS_PROGRESS    = 8
} MAIL_STATUS;

typedef enum
{
    MAIL_DLL_SUCCESS    = 0,
    MAIL_DLL_PROCESSED  = 1,
    MAIL_DLL_REJECT     = 2
} MAIL_RETURN;

typedef struct mail_context_struct
{
    DWORD size;
    DWORD version;
    MAIL_HANDLE context;
    int depth;
    char messageFile[ MAX_FILENAME ];
    char source[ MAX_EMAIL_ADDRESS ];
    char destination[ MAX_EMAIL_ADDRESS ];
} MAIL_CONTEXT;

typedef enum
{
    MAIL_CONFIG_ACTION_NEW_USER     = 1,
    MAIL_CONFIG_ACTION_EDIT_USER    = 2,
    MAIL_CONFIG_ACTION_DELETE_USER  = 4,
    MAIL_CONFIG_ACTION_INSTALL      = 8,
    MAIL_CONFIG_ACTION_UNINSTALL    = 16
} MAIL_CONFIG_ACTION;

typedef struct mail_config_struct
{
    DWORD size;
    DWORD version;
    MAIL_CONFIG_ACTION action;
    MAIL_HANDLE context;
    char user[ MAX_EMAIL_ADDRESS ];
    char domain[ MAX_EMAIL_ADDRESS ];
} MAIL_CONFIG;

typedef int (FAR PASCAL *LPFN_GET_EXTENSION_VERSION)
(
    MAIL_VERSION *data
);

typedef MAIL_RETURN (FAR PASCAL *LPFN_MAIL_EXTENSION)
(
    MAIL_CONTEXT *data
);

typedef int (FAR PASCAL *LPFN_CONFIGURE_EXTENSION)
(
    MAIL_CONFIG *data
);

typedef struct _mail_dll_ifc_struct
{
    int (*deliverMessage)( const char *messageFile, const char *mailFrom, const char *rcptTo, int depth );
    int (*writeToLog)( int status, int logID, const char *message );
    int (*parseMessage)( const char *messageFileIn, const char *templateFile, const char *messageFileOut );
    int (*addVariable)( const char *variableName, const char *value );
    int (*getVariable)( const char *variableName, char *buffer, size_t size );
    int (*setVariable)( const char *variableName, const char *buffer );
    int (*readClient)( void *buffer, size_t size );
    int (*writeClient)( void *buffer, size_t size );
    int (*writeToMessageLog) (const char *mesLog, const char *messageFile, const char *mailFrom, const char *rcptTo, const char *from, const char *to);
    int (*verifyUserPassword) (const char *address, const char *password);
    int (*queueMessage)( const char *messageFile, const char *mailFrom, const char *rcptTo, const char *queue );
    int (*getFilename)( const char *address, const char *filename, char *fullFilename, size_t size );
    int (*sendNotification)( int type, const char *message );
} MAIL_IFC;

typedef struct _mail_dll_info_struct
{
    HINSTANCE hDLL;
    MAIL_HANDLE context;
    MAIL_TYPE type;
    LPFN_GET_EXTENSION_VERSION getExtensionVersion;
    LPFN_MAIL_EXTENSION mailExtension;
    LPFN_CONFIGURE_EXTENSION configureExtension;
} MAIL_INFO;

#define MailDLLVersion (((VER_MAJOR*100)+VER_MINOR)*10000+VER_TRIVIAL)

#ifdef __cplusplus
}
#endif

#endif
