/*
 *
 * Copyright  :: 1995-2002 (C)opyright Gordano Ltd
 *               All rights reserved.
 *               No copies of this source may be made without express written
 *               permission of Gordano Ltd, PO Box 79, Clevedon,
 *               North Somerset, BS21 7TQ, UK
 *
 * $Header: d:\source\data\ntmail17_01\optpack\smtpdll/smtpdll.c,v 1.15 2006/09/19 09:01:16 simont Exp $
 *
 * $Log: smtpdll.c,v $
 * Revision 1.15  2006/09/19  09:01:16  simont
 * VS 2005
 *
 * Revision 1.14  2003/12/30  13:45:49  garrardc
 * fixed main
 *
 * Revision 1.13  2002/07/01  18:19:57  garrardc
 * handle registry update from InternetShopper\Mail to Gordano\Mail
 *
 * Revision 1.12  2002/06/19  14:37:02  garrardc
 * rationalise registry key names
 *
 * Revision 1.11  2002/02/12  12:59:49  garrardc
 * move pragmas
 *
 * Revision 1.10  1999/09/15  17:03:04  garrardc
 * confgured for publication
 *
 * Revision 1.9  1999/09/14  15:27:57  garrardc
 * update include files and copyright
 *
 * Revision 1.8  1999/03/09  13:40:45  garrardc
 * correct log parameter sequences
 *
 * Revision 1.7  1998/04/11  13:38:17  garrardc
 * ensure BaseDir is terminated with '\\'
 * fix variable types
 *
 * Revision 1.6  1998/03/08  17:06:15  garrardc
 * use Rcpt buffer to hold current Rcpt in DataAfterEachRcpt
 *
 * Revision 1.5  1998/03/08  16:59:03  garrardc
 * add DataAfterEchRcpt
 *
 * Revision 1.4  1998/03/06  15:12:58  garrardc
 * consolidate GetUserVariable / GetServerVariable as GetVariable
 * add more example code
 *
 * Revision 1.3  1998/03/06  11:22:15  garrardc
 * include more sample code
 *
 * Revision 1.2  1998/03/06  10:45:47  garrardc
 * add demo ESMTP extension
 *
 * Revision 1.1  1998/03/05  17:56:16  garrardc
 * Initial revision
 *
 * 
 */

#if defined( WINNT ) || defined( WIN95 )
#pragma warning ( disable : 4115 )
#include <windows.h>
#include <winbase.h>
#pragma warning ( disable : 4115 4100 )
#else
#include "windows.h"
#endif

#ifndef MAIN
#define MAIN
#endif

#include "smtp_dll_type.h"
#include "mail_dll_ifc.h"

#define S_SUCCESS 0

#define VER_MAJOR   5
#define VER_MINOR   0
#define VER_TRIVIAL 0

#define CONF_LOG_STATS 0x000010

#define KEY_INTERNET_SHOPPER_PARAMETER     "SOFTWARE\\Gordano\\Mail\\Parameters"

const char *Copyright = "Copyright Gordano Ltd";

static int init = FALSE;
static char BaseDir[MAX_FILENAME];
static char Name[MAX_FILENAME];
static char Description[MAX_FILENAME];

void Initialise(HANDLE hMod)
{
	if (!init)
	{
		HKEY hKey;
		wchar_t Line[MAX_LINE];
		unsigned int size = sizeof(Line);
		char *Buffer;

		GetModuleFileName(hMod, Name, sizeof(Name));
		if (GetFileVersionInfo(Name, 0, sizeof(Line), (void *)Line))
		{
			VerQueryValue(Line, TEXT("\\StringFileInfo\\080904b0\\FileDescription"), (void **)&Buffer, &size);
			strcpy(Description, Buffer);
		}
		else
		{
			size = GetLastError();
		}

		*BaseDir = '\0';
		if (RegCreateKey(HKEY_LOCAL_MACHINE, KEY_INTERNET_SHOPPER_PARAMETER, &hKey) == ERROR_SUCCESS)
		{
			DWORD linelen = sizeof(BaseDir);
			DWORD type_of_entry = REG_SZ;

			RegQueryValueEx(hKey, "BaseDir", 0, &type_of_entry, (unsigned char *)BaseDir, &linelen);
			if (*(BaseDir + strlen(BaseDir) - 1) != '\\')
				strcat(BaseDir, "\\");

			RegCloseKey(hKey);
		}

		init = TRUE;
	}
}

BOOL __stdcall DllMain(HMODULE hmod, ULONG Reason, LPVOID Reserved)
{
	if (Reason == DLL_PROCESS_ATTACH)
	{
		Initialise(hmod);
	}
	else if (Reason == DLL_THREAD_ATTACH)
	{
	}
	else if (Reason == DLL_THREAD_DETACH)
	{
	}
	else if (Reason == DLL_PROCESS_DETACH)
	{
	}

	return (TRUE);
}

int FAR PASCAL GetSMTPExtensionVersion(SMTP_VERSION * data)
{
	int status = S_SUCCESS;
	char Line[MAX_LINE];

	Initialise(NULL);

	wsprintf(Line, "GetSMTPExtensionVersion called in %s - %s (%d)", Name, Description, status);
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	data->version = SMTPDLLVersion;
	strcpy(data->description, Description);
	strcpy(data->copyright, Copyright);

	return (status);
}

int FAR PASCAL ConfigureSMTPExtension(SMTP_CONFIG * data)
{
	int status = S_SUCCESS;
	char Line[MAX_LINE];

	Initialise(NULL);

	wsprintf(Line, "ConfigureSMTPExtension called in %s (%d)", Name, status);
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	return (status);
}

SMTP_RETURN FAR PASCAL SMTPConnection(int id, SMTP_INFO * data)
{
	SMTP_RETURN status = SMTP_DLL_SUCCESS;
	char Line[MAX_LINE];

	Initialise(NULL);

	wsprintf(Line, "%d SMTPConnection called in %s (%d)", id, Name, status);
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	return (status);
}

// example showing creation of private ESMTP extension
SMTP_RETURN FAR PASCAL SMTPEhlo(int id, SMTP_INFO * data)
{
	SMTP_RETURN status = SMTP_DLL_SUCCESS;
	char Line[MAX_LINE];

	Initialise(NULL);

	wsprintf(Line, "%d SMTPEhlo called in %s (%d)", id, Name, status);
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	if (*(data->ESMTPOptions) != '\0')
		strcat(data->ESMTPOptions, ",X-NTMAILEXTENSION");

	return (status);
}

SMTP_RETURN FAR PASCAL SMTPHelo(int id, SMTP_INFO * data)
{
	SMTP_RETURN status = SMTP_DLL_SUCCESS;
	char Line[MAX_LINE];

	Initialise(NULL);

	wsprintf(Line, "%d SMTPHelo called in %s (%d)", id, Name, status);
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	return (status);
}

// example showing simple processing of data in MAIL command
SMTP_RETURN FAR PASCAL SMTPMail(int id, SMTP_INFO * data)
{
	SMTP_RETURN status = SMTP_DLL_SUCCESS;
	char Line[MAX_LINE];
	char user[MAX_LINE];
	char domain[MAX_LINE];
	char Variable[MAX_LINE];
	char VariableName[MAX_LINE];
	char *sep;

	Initialise(NULL);

	wsprintf(Line, "%d SMTPMail called in %s (%d)", id, Name, status);
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	strcpy(Line, data->From);
	sep = strchr(Line, '@');
	if (sep)
	{
		*sep = '\0';
		strcpy(domain, sep + 1);
		sep = domain;
		while (*sep && (*sep == '.' || isalnum(*sep)))
			sep++;
		*sep = '\0';

		sep = Line + strlen(Line) - 1;
		while (sep > Line && (*sep == '.' || isalnum(*sep)))
			sep--;
		strcpy(user, sep);

		*Variable = '\0';
		strcpy(VariableName, "\\\\SMTPLog");
		GetVariable(data->context, VariableName, Variable, sizeof(Variable));
		if (*Variable == '\0')
			wsprintf(Line, "%d SMTPMail %s not found", id, VariableName);
		else
			wsprintf(Line, "%d SMTPMail %s %d", id, VariableName, atoi(Variable));
		WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

		*Variable = '\0';
		wsprintf(VariableName, "%s\\\\IPAddress", domain);
		GetVariable(data->context, VariableName, Variable, sizeof(Variable));
		if (*Variable == '\0')
			wsprintf(Line, "%d SMTPMail %s not found", id, VariableName);
		else
			wsprintf(Line, "%d SMTPMail %s %s", id, VariableName, Variable);
		WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

		*Variable = '\0';
		wsprintf(VariableName, "%s\\%s\\MailBox", domain, user);
		GetVariable(data->context, VariableName, Variable, sizeof(Variable));
		if (*Variable == '\0')
			wsprintf(Line, "%d SMTPMail %s not found", id, VariableName);
		else
			wsprintf(Line, "%d SMTPMail user %s@%s local - %s", id, user, domain, Variable);
		WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

		*Variable = '\0';
		wsprintf(VariableName, "%s\\%s\\DateCreated", domain, user);
		GetVariable(data->context, VariableName, Variable, sizeof(Variable));
		if (*Variable != '\0')
			wsprintf(Line, "%d SMTPMail user %s@%s local - created %s", id, user, domain, Variable);
	}
	else
	{
		wsprintf(Line, "%d SMTPMail invalid address \"%s\"", id, data->From);
	}
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	return (status);
}

// example showing simple processing of data in RCPT command
SMTP_RETURN FAR PASCAL SMTPRcpt(int id, SMTP_INFO * data)
{
	SMTP_RETURN status = SMTP_DLL_SUCCESS;
	char Line[MAX_LINE];
	char user[MAX_LINE];
	char domain[MAX_LINE];
	char Variable[MAX_LINE];
	char VariableName[MAX_LINE];
	char *sep;

	Initialise(NULL);

	wsprintf(Line, "%d SMTPRcpt called in %s (%d)", id, Name, status);
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	strcpy(Line, data->Rcpt);
	sep = strchr(Line, '@');
	if (sep)
	{
		*sep = '\0';
		strcpy(domain, sep + 1);
		sep = domain;
		while (*sep && (*sep == '.' || isalnum(*sep)))
			sep++;
		*sep = '\0';

		sep = Line + strlen(Line) - 1;
		while (sep > Line && (*sep == '.' || isalnum(*sep)))
			sep--;
		strcpy(user, sep);

		*Variable = '\0';
		wsprintf(VariableName, "%s\\%s\\DateCreated", domain, user);
		GetVariable(data->context, VariableName, Variable, sizeof(Variable));
		if (*Variable != '\0')
			wsprintf(Line, "%d SMTPRcpt user %s@%s local - created %s", id, user, domain, Variable);
	}
	else
	{
		wsprintf(Line, "%d SMTPRcpt invalid address \"%s\"", id, data->From);
	}
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	return (status);
}

SMTP_RETURN FAR PASCAL SMTPDataBefore(int id, SMTP_INFO * data)
{
	SMTP_RETURN status = SMTP_DLL_SUCCESS;
	char Line[MAX_LINE];

	Initialise(NULL);

	wsprintf(Line, "%d SMTPDataBefore called in %s (%d)", id, Name, status);
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	return (status);
}

SMTP_RETURN FAR PASCAL SMTPDataAfter(int id, SMTP_INFO * data)
{
	SMTP_RETURN status = SMTP_DLL_SUCCESS;
	char Line[MAX_LINE];

	Initialise(NULL);

	wsprintf(Line, "%d SMTPDataAfter called in %s (%d)", id, Name, status);
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	return (status);
}

SMTP_RETURN FAR PASCAL SMTPDataAfterEachRcpt(int id, SMTP_INFO * data)
{
	SMTP_RETURN status = SMTP_DLL_SUCCESS;
	char Line[MAX_LINE];

	Initialise(NULL);

	wsprintf(Line, "%d SMTPDataAfterEachRcpt called in %s (%d)", id, Name, status);
	WriteToLog(data->context, CONF_LOG_STATS, 3999, Line);

	return (status);
}
