// Name: Mouse Gestures
// Description: Mouse gestures with right click
// AccessRights: ALL
// ConfigurationRequired: Yes


var gestureSelf;

function gmsPlugin_gesture()
{
    gestureSelf = this;

    gestureSelf.gestureAllowed = false;
    gestureSelf.gestureRight = false;
    gestureSelf.gestureX = -1;
    gestureSelf.gestureY = -1;
}

gmsPlugin_gesture.prototype.handleemail = function() { return true; };
gmsPlugin_gesture.prototype.handleaddressbook = function() { return true; };
gmsPlugin_gesture.prototype.handlecalendar = function() { return true; };
gmsPlugin_gesture.prototype.handletasks = function() { return true; };
gmsPlugin_gesture.prototype.handlenotes = function() { return true; };

function gestureEventStopPropagation(ev)
{
    if (ev.stopPropagation)
    {
        ev.stopPropagation();
    }
    else
    {
        ev.cancelBubble = true;
    }
}

function gestureEventPreventDefault(ev)
{
    if (ev.preventDefault)
    {
        ev.preventDefault();
    }
    else
    {
        var el = ev.target || ev.srcElement;

        ev.returnValue = false;

        if (el.tagName.toLowerCase () != "input")
            ev.keyCode = 0;
    }
}

function gestureHandleMouseDown(e)
{
    var allowed = true;
    var obj = null;

    obj = e.target;

    if (!obj)
    {
        obj = e.srcElement;
    }

    if (obj)
    {
        if (obj.id == "attachment")
        {
            allowed = false;
        }
    }

    gestureSelf.gestureAllowed = allowed;

    if (allowed)
    {
        if (e.button == 2)
        {
            gestureEventPreventDefault (e);
            gestureEventStopPropagation (e);

            gestureSelf.gestureRight = true;
            gestureSelf.gestureX = -1;
            gestureSelf.gestureY = -1;

            return false;
        }
        else
        {
            gestureSelf.gestureRight = false;
        }
    }
    else
    {
        gestureSelf.gestureRight = false;
    }
}

function gestureHandleMouseUp(e)
{
    if (gestureSelf.gestureRight)
    {
        gestureEventPreventDefault (e);
        gestureEventStopPropagation (e);

        gestureSelf.gestureRight = false;
        gestureSelf.gestureX = -1;
        gestureSelf.gestureY = -1;

        return false;
    }
}

function gestureMouseMove(e)
{
    var udil = null;
    var action = null;

    if (gestureSelf.gestureRight)
    {
        gestureEventPreventDefault (e);
        gestureEventStopPropagation (e);

        if (gestureSelf.gestureX == -1)
        {
            gestureSelf.gestureX = e.clientX;
        }

        if (gestureSelf.gestureY == -1)
        {
            gestureSelf.gestureY = e.clientY;
        }

        if (e.clientX - gestureSelf.gestureX  > 25)
        {
            if (gestureSelf.type == "EMAIL")
            {
                action = "replyall";
            }
            else
            {
                action = "open";
            }
        }
        else if (gestureSelf.gestureX - e.clientX > 25)
        {
            action = "delete";
        }
        else if (e.clientY - gestureSelf.gestureY  > 25)
        {
            if (gestureSelf.existMenuCommand("readnext"))
            {
                action = "readnext";
            }
            else
            {
                udil = gestureSelf.getNextValue();
            }
        }
        else if (gestureSelf.gestureY - e.clientY > 25)
        {
            if (gestureSelf.existMenuCommand("readprev"))
            {
                action = "readprev";
            }
            else
            {
                udil = gestureSelf.getPreviousValue();
            }
        }

        if (udil != null && udil != "")
        {
            gestureSelf.gestureRight = false;
            gestureSelf.gestureX = -1;
            gestureSelf.gestureY = -1;

            gestureSelf.selectValue(udil);

        }
        else if (action != null && action != "")
        {
            gestureSelf.gestureRight = false;
            gestureSelf.gestureX = -1;
            gestureSelf.gestureY = -1;

            gestureSelf.clickMenuCommand(action);
        }
    }
    else
    {
        gestureSelf.gestureRight = false;
        gestureSelf.gestureX = e.clientX;
        gestureSelf.gestureY = e.clientY;
    }
}

gmsPlugin_gesture.prototype.previewload = function()
{
    var s = this.getSummaryWindow();
    var v = this.getViewerWindow();

    s.document.onmousedown = function(e) { return gestureHandleMouseDown (e || s.event); }
    v.document.onmousedown = function(e) { return gestureHandleMouseDown (e || v.event); }

    s.document.onmouseup = function(e) { return gestureHandleMouseUp (e || s.event); }
    v.document.onmouseup = function(e) { return gestureHandleMouseUp (e || v.event); }

    s.document.onmousemove = function(e) { return gestureMouseMove (e || s.event); };
    v.document.onmousemove = function(e) { return gestureMouseMove (e || v.event); };

    s.document.body.oncontextmenu = function(e) { if (gestureSelf.gestureAllowed) { (e || s.event).cancelBubble=true; return false; } };
    v.document.body.oncontextmenu = function(e) { if (gestureSelf.gestureAllowed) { (e || v.event).cancelBubble=true; return false; } };
}

gmsPlugin_gesture.prototype.configload = function()
{
    var html = "";

    html += "<center>Example Mouse Gesture Gizmo</center><br>";
    html += "<center>";
    html += "<table>";
    html += "<tr><td colspan=2>The following gestures are supported:</td></tr>";
    html += "<tr><td colspan=2></td></tr>";
    html += "<tr><td>Left</td><td>Delete</td></tr>";
    html += "<tr><td>Right</td><td>Reply All or Open</td></tr>";
    html += "<tr><td>Up</td><td>Previous</td></tr>";
    html += "<tr><td>Down</td><td>Next</td></tr>";
    html += "</table>";

    this.insertCustomContent("config", html);
}

