// Name: Twitter
// Description: Twitter Client
// AccessRights: ALL
// ConfigurationRequired: Yes

if( document.implementation.hasFeature("XPath", "3.0") ) 
{ 
	XMLDocument.prototype.selectNodes = function(cXPathString, xNode) 
	{ 
		if( !xNode ) { xNode = this; }
		
		var oNSResolver = this.createNSResolver(this.documentElement) 
		var aItems = this.evaluate(cXPathString, xNode, oNSResolver,
		
		XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null) 
		var aResult = [];
		for( var i = 0; i < aItems.snapshotLength; i++) 
		{ 
			aResult[i] = aItems.snapshotItem(i);
		} 
		return aResult;
	} 

	Element.prototype.selectNodes = function(cXPathString) 
	{ 
		if(this.ownerDocument.selectNodes) 
		{ 
			return this.ownerDocument.selectNodes(cXPathString, this);
		} 
		else
		{
			throw "For XML Elements Only";
		} 
	} 
}

var twitterSelf;

function gmsPlugin_twitter()
{
    twitterSelf = this;

    twitterSelf.twitterTreeResult = "";

    twitterSelf.twitterIdResultCache = new Object();
    twitterSelf.twitterTimelineResultCache = new Object();

    twitterSelf.twitterWindow = "";
}

gmsPlugin_twitter.prototype.handleemail = function() { return false; };
gmsPlugin_twitter.prototype.handleaddressbook = function() { return false; };
gmsPlugin_twitter.prototype.handlecalendar = function() { return false; };
gmsPlugin_twitter.prototype.handletasks = function() { return false; };
gmsPlugin_twitter.prototype.handlenotes = function() { return false; };

function twitterConfigured()
{
    var isConfigured = false;

    if (twitterSelf.getConfigurationValue("account", "") != "")
    {
        isConfigured = true;
    }

    return isConfigured;
}

function processTwitterError(responseText)
{
    var xmlDoc;

    try //Internet Explorer
    {
        xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
        xmlDoc.async="false";
        xmlDoc.loadXML(responseText);
    }
    catch(e)
    {
        try //Firefox, Mozilla, Opera, etc.
        {
            parser=new DOMParser();
            xmlDoc=parser.parseFromString(responseText,"text/xml");
        }
        catch(e) {alert(e.message)}
    }

    var errorArray = xmlDoc.getElementsByTagName("error");

    alert(errorArray[0].firstChild.nodeValue);
}

function twitterGetResponseData(xmlDoc, attr, index)
{
    var str = "";
    var tempArray;

    if (index === undefined)
    {
        index = 0;
    }

    tempArray = xmlDoc.getElementsByTagName(attr);

    if (tempArray.length > index)
    {
        if (tempArray[index].firstChild)
        {
            str = tempArray[index].firstChild.nodeValue;
        }
    }

    return str;
}

var twitterMonthNames = new Array(
'January',
'February',
'March',
'April',
'May',
'June',
'July',
'August',
'September',
'October',
'November',
'December'
);

function twitterDateFormat (date)
{
    var dateArray = date.split(' ');

    return new Date(
      Date.parse( dateArray[0] + ', ' + dateArray[2] + ' '
         + dateArray[1] + ' ' + dateArray[3] + ' '
         + dateArray[5].substring(0,4) + " GMT")
    );
}

function twitterGetDateStr(now, date)
{
    var diff;
    var year;
    var weeks;
    var days;
    var hours;
    var mins;
    var secs;
    var typeStr;
    var dateStr;

    diff = (now - date) / 1000;

    if (diff < 0)
    {
        diff = 0;
    }

    weeks = Math.floor(diff / (60 * 60 * 24 * 7));
    diff -= weeks * (60 * 60 * 24 * 7);

    days = Math.floor(diff / (60 * 60 * 24));
    diff -= days * (60 * 60 * 24);

    hours = Math.floor(diff / (60 * 60));
    diff -= hours * (60 * 60);

    mins = Math.floor(diff / (60));
    diff -= mins * (60);

    secs = Math.floor(diff);
    diff -= secs;

    dateStr = "";

    if (weeks > 4)
    {
        dateStr += date.getDate() + " ";
        dateStr += twitterMonthNames[date.getMonth()] + " ";

        dateStr += date.getFullYear();
    }
    else if (weeks != 0 || days != 0)
    {
        diff = days + 7 * weeks;
        typeStr = "day";
    }
    else if (hours != 0)
    {
        diff = hours;

        if (mins >= 30)
        {
            diff++;
        }

        typeStr = "hour";
    }
    else if (mins != 0)
    {
        diff = mins;
        typeStr = "minute";

        if (secs >= 30)
        {
            diff++;
        }
    }
    else
    {
        diff = secs;
        typeStr = "second";
    }

    if (dateStr == "")
    {
        dateStr += diff + " " + typeStr;

        if (diff != 1)
        {
            dateStr += "s";
        }

        dateStr += " ago";
    }

    return dateStr;
}

function processTwitterTimelineResult(id, responseText)
{
    var html = "";
    var textArray;
    var dateArray;
    var nameArray;
    var imageArray;
    var sourceArray;
    var name;
    var imageUrl;
    var text;
    var date;
    var source;

    var xmlDoc;

    try //Internet Explorer
    {
        xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
        xmlDoc.async="false";
        xmlDoc.loadXML(responseText);
    }
    catch(e)
    {
        try //Firefox, Mozilla, Opera, etc.
        {
            parser=new DOMParser();
            xmlDoc=parser.parseFromString(responseText,"text/xml");
        }
        catch(e) {alert(e.message)}
    }

    var includeName = false;
    var showImages = false;
    var replyEnabled = true;
    var colspan = 1;

    if (twitterConfigured())
    {
        replyEnabled = true;
    }
    else
    {
        replyEnabled = false;
    }

    if (id == twitterSelf.getConfigurationValue("account", ""))
    {
        includeName = true;
        showImages = true;
        replyEnabled = false;
        colspan = 3;
    }

    if (replyEnabled)
    {
        twitterSelf.enableMenuCommand("reply");
    }
    else
    {
        twitterSelf.disableMenuCommand("reply");
    }

    twitterSelf.twitterTimelineResultCache[id] = responseText;
 
    textArray = xmlDoc.selectNodes("/statuses/status/text");
    dateArray = xmlDoc.selectNodes("/statuses/status/created_at");
    nameArray = xmlDoc.selectNodes("/statuses/status/user/screen_name");
    imageArray = xmlDoc.selectNodes("/statuses/status/user/profile_image_url");
    sourceArray = xmlDoc.selectNodes("/statuses/status/source");
    /* jc 22-May-2009 - Implemented selectnodes */

    html += "<table width=100% cellspacing=0 cellpadding=0>";

    var now = new Date();

    var index;
    var url;
    var dateStr;
    var date;
    var temp;

    for (var i=0;i<textArray.length;i++)
    {
        dateStr = "";

        text = textArray[i].firstChild.nodeValue;
        name = nameArray[i].firstChild.nodeValue;
        imageUrl = imageArray[i].firstChild.nodeValue;
        source = sourceArray[i].firstChild.nodeValue;

        dateStr = dateArray[i].firstChild.nodeValue;
        date = twitterDateFormat(dateStr);

        dateStr = twitterGetDateStr(now, date);

        html += "<tr height=8><td colspan=" + colspan + "><hr></td></tr>";
        html += "<tr>";

        if (showImages)
        {
            html += "<td width=1%><img src=\"" + imageUrl + "\" border=0></td><td>&nbsp;&nbsp;&nbsp;</td>";
        }

        html += "<td valign=top>";

        if (includeName)
        {
            html += "<b>" + name + "</b>&nbsp;";
        }

        index = text.indexOf("http:");

        if (index == -1)
        {
            index = text.indexOf("https:");
        }

        text = text.replace(/\n/g, " ");
        if (index != -1)
        {
            temp = text.slice(0, index);

            text = text.slice(index);

            index = text.indexOf(" ");

            if (index != -1)
            {
                url = text.slice(0, index);
                text = text.slice(index);
            }
            else
            {
                url = text;
                text = "";
            }

            text = temp + "<a target=_blank href=\"" + url + "\">link</a>" + text;
        }

        html += text;
        html+= "&nbsp;" + dateStr;

        source = source.replace(/<a /g, "<a target=\"_blank\" ");

        html += "&nbsp;from " + source;

        html += "</td>";

        html += "</tr>";
    }

    html += "<tr height=8><td colspan=" + colspan + "><hr></td></tr>";

    html += "</table>";
 
    twitterSelf.insertCustomContent("timeline", html);
}

function twitterTimelineCallback(details)
{
    if (details.status == 200)
    {
        if (details.responseText != "")
        {
            processTwitterTimelineResult(details.uniqueid, details.responseText);
        }
    }
    else
    {
        processTwitterError(details.responseText);
    }
}

function twitterTimeout(responseDetails)
{
    alert("Twitter timeout " + responseDetails.url);
}

function twitterLoadUserTimeline()
{
    var id = twitterSelf.getFieldValue("id");

    if (twitterSelf.twitterTimelineResultCache[id] === undefined)
    {
        var account = "";
        var password = "";

        var count = twitterSelf.getConfigurationValue("count", "20");

        id = id.toLowerCase();
        account = twitterSelf.getConfigurationValue("account", "");
        account = account.toLowerCase();

        if (id == account)
        {
            url = "http://twitter.com/statuses/friends_timeline.xml?count=" + count;

            password = twitterSelf.getConfigurationValue("password", "", true);
        }
        else
        {
            url = "http://twitter.com/statuses/user_timeline.xml?id=" + id + "&count=" + count;

            account = "";
        }

        var details = new Object();

        details.url = url;
        details.method = "GET";
        details.headers = {};
        details.data = "";
        details.authtype = "Basic";
        details.account = account;
        details.password = password;
        details.onload = twitterTimelineCallback;
        details.onerror = twitterTimelineCallback;
        details.uniqueid = id;
        details.timeout = twitterSelf.getConfigurationValue("timeout", "5");
        details.ontimeout = twitterTimeout;

        twitterSelf.xmlhttpRequest(details);
    }
    else
    {
        processTwitterTimelineResult(id, twitterSelf.twitterTimelineResultCache[id]);
    }
}

function processTwitterIdResult(responseText)
{
    var html = "";
    var enabled = false;

    if (twitterConfigured())
    {
        enabled = true;
    }

    twitterSelf.registerMenuCommand("+Reply", "Ctrl+Shift+R", "Reply", "twitter-reply.png", "reply", false, handleMenuClick);
    twitterSelf.addMenuDivider();
    twitterSelf.registerMenuCommand("Refresh", "Ctrl+Shift+R", "Refresh", "refresh.png", "refresh", enabled, handleMenuClick);
    twitterSelf.addMenuDivider();

    try //Internet Explorer
    {
        xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
        xmlDoc.async="false";
        xmlDoc.loadXML(responseText);
    }
    catch(e)
    {
        try //Firefox, Mozilla, Opera, etc.
        {
            parser=new DOMParser();
            xmlDoc=parser.parseFromString(responseText,"text/xml");
        }
        catch(e) {alert(e.message)}
    }

    var name = twitterGetResponseData(xmlDoc, "name");
    var id = twitterGetResponseData(xmlDoc, "screen_name");
    var location = twitterGetResponseData(xmlDoc, "location");
    var url = twitterGetResponseData(xmlDoc, "url");
    var imageUrl = twitterGetResponseData(xmlDoc, "profile_image_url");
    var description = twitterGetResponseData(xmlDoc, "description");

    twitterSelf.twitterIdResultCache[id] = responseText;

    html += "<html>";
    html += "<head><link rel=\"stylesheet\" href=\"/webmail/plugins/twitter.css\" type=\"text/css\"></head>";
    html += "<body>";
    html += "<input type=hidden id=\"id\" value=\"" + id + "\">";
    html += "<table width=90% align=center cellspacing=0 cellpadding=0>";
    html += "<tr><td>" + name + "</td><td align=right>";
    html += "</body></html>";

    if (url != "")
    {
        html += "<a href=\"" + url + "\" target=_blank>";
    }

    html += "<img src=\"" + imageUrl + "\" border=0>";

    if (url != "")
    {
        html += "</a>";
    }

    html += "</td></tr>";

    html += "<tr><td colspan=2>&nbsp;</td></tr>";

    if (location != "")
    {
        html += "<tr height=8><td colspan=2></td></tr>";
        html += "<tr><td colspan=2><b>Location</b>&nbsp;" + location + "</td></tr>";
    }

    if (url != "")
    {
        html += "<tr height=8><td colspan=2></td></tr>";
        html += "<tr><td colspan=2><b>Web</b>&nbsp;<a href=\"" + url + "\" target=_blank>" + url + "</a></td></tr>";
    }

    if (description != "")
    {
        html += "<tr height=8><td colspan=2></td></tr>";
        html += "<tr><td colspan=2><b>Bio</b>&nbsp;" + description + "</td></tr>";
    }

    html += "<tr><td colspan=2>&nbsp;</td></tr>";
    html += "<tr><td colspan=2><div id=\"timeline\"></div></td></tr>";
    html += "</table>";
    html += "</body></html>";

    twitterSelf.loadCustomContent(name, html, "twitterLoadUserTimeline", false);
}

function twitterIdCallback(details)
{
    if (details.status == 200)
    {
        processTwitterIdResult(details.responseText);
    }
    else
    {
        processTwitterError(details.responseText);
    }
}

function twitterWindowCallback(w, parameters)
{
    var html = "";

    if (parameters != "")
    {
        parameters = "@" + parameters + " ";
    }

    var enabled = false;

    if (twitterConfigured())
    {
        enabled = true;
    }

    twitterSelf.removeMenuCommand("help", "admin", w);
    twitterSelf.registerMenuCommand("+Twitter", "Ctrl+Enter", "Twitter", "twitter.png", "sendtwitter", enabled, handleMenuClick, "command", w);

    html += "<table width=90% align=center cellspacing=0 cellpadding=0>";
    html += "<tr><td colspan=2>&nbsp;</td></tr>";
    html += "<tr><td><b>What are you doing?</b></td></tr>";
    html += "<tr><td colspan=2>&nbsp;</td></tr>";
    html += "<tr><td><input id=\"twitter\" size=40 maxlength=140 value=\"" + parameters + "\"></td></tr>";
    html += "</table>";

    twitterSelf.insertCustomContent("content", html, w);
}

function twitterSendCallback(details)
{
    if (details.status == 200)
    {

    }
    else
    {
        processTwitterError(details.responseText);
    }
}

function twitterSend(text)
{
    url = "http://twitter.com/statuses/update.xml?status=" + urlencode(text) + "&source=gordanomessagingsuite";

    var details = new Object();

    details.url = url;
    details.method = "POST";
    details.headers = {};
    details.data = "";
    details.authtype = "Basic";
    details.account = twitterSelf.getConfigurationValue("account", "");
    details.password = twitterSelf.getConfigurationValue("password", "", true);
    details.onload = twitterSendCallback;
    details.onerror = twitterSendCallback;
    details.timeout = twitterSelf.getConfigurationValue("timeout", "5");
    details.ontimeout = twitterTimeout;

    twitterSelf.xmlhttpRequest(details);
}

function handleMenuClick(action)
{
    if (!twitterConfigured())
    {
        alert("You need to configure your Twitter account before using this Gizmo");
    }
    else if (action == "twitter")
    {
        if (twitterSelf.twitterWindow == "" || twitterSelf.twitterWindow.closed)
        {
            twitterSelf.twitterWindow = twitterSelf.openCustomWindow("twitterWindowCallback", "", 100, 250);
        }
        else
        {
            twitterSelf.twitterWindow.focus();
        }
    }
    else if (action == "reply")
    {
        var id = twitterSelf.getFieldValue("id");

        if (twitterSelf.twitterWindow == "" || twitterSelf.twitterWindow.closed)
        {
            twitterSelf.twitterWindow = twitterSelf.openCustomWindow("twitterWindowCallback", id, 100, 250);
        }
        else
        {
            twitterSelf.twitterWindow.focus();
        }
    }
    else if (action == "refresh")
    {
        var str = twitterSelf.getSelectedTreeNode();

        var id;
        var params = str.split(".");

        if (params.length == 1)
        {
            id = twitterSelf.getConfigurationValue("account", "");
        }
        else if (params.length == 2)
        {
            id = params[1];
        }

        twitterSelf.twitterIdResultCache[id] = undefined;
        twitterSelf.twitterTimelineResultCache[id] = undefined;

        handleTreeClick(str, "");
    }
    else if (action == "sendtwitter")
    {
        var value = twitterSelf.getFieldValue("twitter", twitterSelf.twitterWindow);

        if (value != "")
        {
            twitterSelf.twitterWindow.close();

            twitterSend(value);

            twitterSelf.twitterWindow = "";
        }
        else
        {
            twitterSelf.twitterWindow.alert("Please enter your Tweet");
        }
    }
}

function handleTreeClick(id, lastId)
{
    if (twitterConfigured())
    {
        var params = id.split(".");

        if (params.length == 1)
        {
            id = twitterSelf.getConfigurationValue("account", "");
        }
        else if (params.length == 2)
        {
            id = params[1];
        }

        if (twitterSelf.twitterIdResultCache[id] === undefined)
        {
            url = "http://twitter.com/users/show/" + id + ".xml";

            var details = new Object();

            details.url = url;
            details.method = "GET";
            details.headers = {};
            details.data = "";
            details.authtype = "Basic";
            details.account = twitterSelf.getConfigurationValue("account", "");
            details.password = twitterSelf.getConfigurationValue("password", "", true);
            details.onload = twitterIdCallback;
            details.onerror = twitterIdCallback;
            details.timeout = twitterSelf.getConfigurationValue("timeout", "5");
            details.ontimeout = twitterTimeout;

            twitterSelf.xmlhttpRequest(details);
        }
        else
        {
            processTwitterIdResult(twitterSelf.twitterIdResultCache[id]);
        }
    }
    else
    {
        var html = "";

        html += "<html>";
        html += "<head><link rel=\"stylesheet\" href=\"/webmail/plugins/twitter.css\" type=\"text/css\"></head>";
        html += "<body>";
        html += "<br><br><center>You need to configure your Twitter account before using this Gizmo";
        html += "</body></html>";

        twitterSelf.loadCustomContent("Twitter", html, "", false);
    }
}

function processTwitterTreeResult()
{
    var xmlDoc;

    try //Internet Explorer
    {
        xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
        xmlDoc.async="false";
        xmlDoc.loadXML(twitterSelf.twitterTreeResult);
    }
    catch(e)
    {
        try //Firefox, Mozilla, Opera, etc.
        {
            parser=new DOMParser();
            xmlDoc=parser.parseFromString(twitterSelf.twitterTreeResult,"text/xml");
        }
        catch(e) {alert(e.message)}
    }

    var namesArray = xmlDoc.getElementsByTagName("name");
    var screenNamesArray = xmlDoc.getElementsByTagName("screen_name");

    var name;
    var screenName;

    for(var i=0;i<namesArray.length;i++)
    {
        name = namesArray[i].firstChild.nodeValue;
        screenName = screenNamesArray[i].firstChild.nodeValue;

        twitterSelf.addTreeNodeChild(name, "twitter.png", "twitter." + screenName, "twitter", handleTreeClick, "twitter");
    }

    if (twitterSelf.getCurrentPanel() != "twitter" && twitterSelf.getSelectedTreeNode() != "twitter")
    {
        twitterSelf.closeTreeNode("twitter");
    }
}

function twitterCallback(details)
{
    if (details.status == 200)
    {
        twitterSelf.twitterTreeResult = details.responseText;

        processTwitterTreeResult();
    }
    else
    {
        processTwitterError(details.responseText);
    }
}

function loadTwitterTree()
{
    if (twitterSelf.twitterTreeResult == "")
    {
        var details = new Object();

        details.url = "http://twitter.com/statuses/friends.xml";
        details.method = "GET";
        details.headers = {};
        details.data = "";
        details.authtype = "Basic";
        details.account = twitterSelf.getConfigurationValue("account", "");
        details.password = twitterSelf.getConfigurationValue("password", "", true);
        details.onload = twitterCallback;
        details.onerror = twitterCallback;
        details.timeout = twitterSelf.getConfigurationValue("timeout", "5");
        details.ontimeout = twitterTimeout;

        twitterSelf.xmlhttpRequest(details);
    }
    else
    {
        processTwitterTreeResult();
    }
}

gmsPlugin_twitter.prototype.menuload = function()
{
    var enabled = false;

    if (twitterConfigured())
    {
        enabled = true;
    }

    this.registerMenuCommand("Twitter", "Ctrl+Shift+T", "Twitter", "twitter.png", "twitter", enabled, handleMenuClick, "command");

    this.addMenuDivider("command");
}

gmsPlugin_twitter.prototype.treeload = function()
{
    this.addTreeNodeNext("Twitter", "Twitter.png", "twitter", "twitter", handleTreeClick);

    if (twitterConfigured())
    {
        loadTwitterTree();
    }
}

gmsPlugin_twitter.prototype.panelload = function()
{
    this.addPanelButton("Twitter", "", "Twitter.png", "twitter");
}

function twitterConfigButtonHandler(action)
{
    var ok = true;
    var reload = false;

    var account = twitterSelf.getFieldValue("account");
    var password = twitterSelf.getFieldValue("password");
    var timeout = twitterSelf.getFieldValue("timeout");
    var count = twitterSelf.getFieldValue("count");

    var old_account = twitterSelf.getConfigurationValue("account", "");
    var old_timeout = twitterSelf.getConfigurationValue("timeout", "5");
    var old_count = twitterSelf.getConfigurationValue("count", "20");

    if (account != "")
    {
        if (old_account != account)
        {
            twitterSelf.setConfigurationValue("account", account);

            reload = true;
        }
    }
    else
    {
        ok = false;

        alert("Please enter your account details");
    }

    if (password != "")
    {
        reload = true;

        twitterSelf.setConfigurationValue("password", password, true);
    }

    if (old_timeout != timeout)
    {
        twitterSelf.setConfigurationValue("timeout", timeout);

        reload = true;
    }

    if (old_count != count)
    {
        if (count > 200)
        {
            count = 200;

            alert("Status count is limited to 200");
        }

        twitterSelf.setConfigurationValue("count", count);

        reload = true;
    }

    if (ok && reload)
    {
        twitterSelf.reload();
    }
}

function twitterRateLimitStatusCallback(details)
{
    if (details.status == 200)
    {
        var xmlDoc;

        try //Internet Explorer
        {
            xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
            xmlDoc.async="false";
            xmlDoc.loadXML(details.responseText);
        }
        catch(e)
        {
            try //Firefox, Mozilla, Opera, etc.
            {
                parser=new DOMParser();
                xmlDoc=parser.parseFromString(details.responseText,"text/xml");
            }
            catch(e) {alert(e.message)}
        }

        var remaining_hits = twitterGetResponseData(xmlDoc, "remaining-hits");
        var hourly_limit = twitterGetResponseData(xmlDoc, "hourly-limit");

        var html = remaining_hits + " remaining of " + hourly_limit

        twitterSelf.insertCustomContent("ratelimit", html);
    }
    else
    {
        processTwitterError(details.responseText);
    }
}

function twitterGetRateLimitStatus()
{
    var details = new Object();

    details.url = "http://twitter.com/account/rate_limit_status.xml";
    details.method = "GET";
    details.headers = {};
    details.data = "";
    details.authtype = "Basic";
    details.account = twitterSelf.getConfigurationValue("account", "");
    details.password = twitterSelf.getConfigurationValue("password", "", true);
    details.onload = twitterRateLimitStatusCallback;
    details.onerror = twitterRateLimitStatusCallback;
    details.timeout = twitterSelf.getConfigurationValue("timeout", "5");
    details.ontimeout = twitterTimeout;

    twitterSelf.xmlhttpRequest(details);
}

gmsPlugin_twitter.prototype.configload = function()
{
    var html = "";

    html += "<table>";
    html += "<tr><td>Account</td><td><input id=\"account\"></td></tr>";
    html += "<tr><td>Password</td><td><input id=\"password\" type=password></td></tr>";
    html += "<tr><td>&nbsp;</td></tr>";
    html += "<tr><td>Status count</td><td><input id=\"count\"></td></tr>";
    html += "<tr><td>Timeout</td><td><input id=\"timeout\">&nbsp;seconds</td></tr>";
    html += "<tr><td>&nbsp;</td></tr>";
    html += "<tr><td>Rate limit status</td><td><span id=\"ratelimit\"></span></td></tr>";
    html += "<tr><td>&nbsp;</td></tr>";
    html += "<tr><td align=right colspan=2><span id=\"button1\"></span></td></tr>";
    html += "</table>";

    this.insertCustomContent("config", html);

    this.insertCustomButton("button1", "     OK     ", "ok", twitterConfigButtonHandler);

    var account = this.getConfigurationValue("account", "");
    var timeout = this.getConfigurationValue("timeout", "5");
    var count = this.getConfigurationValue("count", "20");

    this.setFieldValue("account", account);
    this.setFieldValue("timeout", timeout);
    this.setFieldValue("count", count);

    twitterGetRateLimitStatus();
}
